package de.twomartens.oparlservice.entity;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.annotation.JsonProperty;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.*;

import java.time.ZonedDateTime;
import java.util.List;

@Builder
@Getter
@ToString
@EqualsAndHashCode
@AllArgsConstructor(access = AccessLevel.PRIVATE)
@NoArgsConstructor(force = true, access = AccessLevel.PRIVATE)
@JsonInclude(JsonInclude.Include.NON_NULL)
public class AgendaItem {
    @NonNull
    @Schema(required = true, description = "The unique identifier of the object")
    private final String id;

    @NonNull
    @Schema(required = true, description = "Namespace URL to the object type schema")
    private final String type;

    @NonNull
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @Schema(required = true, description = "Date and time of the creation of the object")
    private final ZonedDateTime created;

    @NonNull
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @Schema(required = true, description = "Date and time of the last update of the object")
    private final ZonedDateTime modified;

    @Schema(description = "URL to the license", nullable = true)
    private String license;

    @Schema(description = "Categorization of the object", nullable = true)
    private String keyword;

    @Schema(description = "URL of a website that presents the object in the browser", nullable = true)
    private String web;

    @Schema(description = "True, if the object was deleted")
    private final boolean deleted;

    @Schema(description = "URL to the meeting this agenda item is part of", nullable = true)
    private final String meeting;

    @Schema(description = "number of the agenda item", nullable = true, example = "10.1, A, ...")
    private final String number;

    @Schema(description = "position of the agenda item in the meeting based on 0", required = true)
    private final int order;

    @Schema(description = "the topic of the agenda item", nullable = true)
    private final String name;

    @JsonProperty("public")
    @Schema(description = "True if the agenda item is public")
    private final boolean isPublic;

    @Schema(description = "URL to the consultation related to this agenda item", nullable = true)
    private final String consultation;

    @Schema(description = "categorical information about result of consultation of this agenda item", nullable = true)
    private final String result;

    @Schema(description = "resolution text of a decision in this agenda item", nullable = true)
    private final String resolutionText;

    @Schema(description = "resolution file of a decision in this agenda item", nullable = true)
    private final File resolutionFile;

    @Schema(description = "additional appendices to this agenda item", nullable = true)
    private final List<File> auxiliaryFile;

    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @Schema(description = "date and time of the begin of the agenda item", nullable = true)
    private final ZonedDateTime start;

    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @Schema(description = "date and time of the end of the agenda item", nullable = true)
    private final ZonedDateTime end;
}
