package de.twomartens.oparlservice.entity;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonInclude;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.*;

import java.time.ZonedDateTime;
import java.util.List;

@Builder
@Getter
@ToString
@EqualsAndHashCode
@AllArgsConstructor(access = AccessLevel.PRIVATE)
@NoArgsConstructor(force = true, access = AccessLevel.PRIVATE)
@JsonInclude(JsonInclude.Include.NON_NULL)
public class Consultation {
    @NonNull
    @Schema(required = true, description = "The unique identifier of the object")
    private final String id;

    @NonNull
    @Schema(required = true, description = "Namespace URL to the object type schema")
    private final String type;

    @NonNull
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @Schema(required = true, description = "Date and time of the creation of the object")
    private final ZonedDateTime created;

    @NonNull
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @Schema(required = true, description = "Date and time of the last update of the object")
    private final ZonedDateTime modified;

    @Schema(description = "URL to the license", nullable = true)
    private String license;

    @Schema(description = "Categorization of the object", nullable = true)
    private String keyword;

    @Schema(description = "URL of a website that presents the object in the browser", nullable = true)
    private String web;

    @Schema(description = "True, if the object was deleted")
    private final boolean deleted;

    @Schema(description = "URL to the paper related to this consultation", nullable = true)
    private final String paper;

    @Schema(description = "URL to the agenda item related to this consultation", nullable = true)
    private final String agendaItem;

    @Schema(description = "URL to the meeting related to this consultation", nullable = true)
    private final String meeting;

    @Schema(description = "List of URLs to organizations that participate in this consultation", nullable = true)
    private final List<String> organization;

    @Schema(description = "True if a decision will be or has been reached in this consultation")
    private final boolean authoritative;

    @Schema(description = "role or function of this consultation", example = "Anhörung, Entscheidung, usw", nullable = true)
    private final String role;
}
