package de.twomartens.oparlservice.entity;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonInclude;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.*;

import java.time.ZonedDateTime;
import java.util.List;

@Builder
@Getter
@ToString
@EqualsAndHashCode
@AllArgsConstructor(access = AccessLevel.PRIVATE)
@NoArgsConstructor(force = true, access = AccessLevel.PRIVATE)
@JsonInclude(JsonInclude.Include.NON_NULL)
public class Location {
    @NonNull
    @Schema(required = true, description = "The unique identifier of the object")
    private final String id;

    @NonNull
    @Schema(required = true, description = "Namespace URL to the object type schema")
    private final String type;

    @NonNull
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @Schema(required = true, description = "Date and time of the creation of the object")
    private final ZonedDateTime created;

    @NonNull
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @Schema(required = true, description = "Date and time of the last update of the object")
    private final ZonedDateTime modified;

    @Schema(description = "URL to the license", nullable = true)
    private String license;

    @Schema(description = "Categorization of the object", nullable = true)
    private String keyword;

    @Schema(description = "URL of a website that presents the object in the browser", nullable = true)
    private String web;

    @Schema(description = "True, if the object was deleted")
    private final boolean deleted;

    @Schema(description = "description of the location", nullable = true)
    private final String description;

    @Schema(description = "GeoJSON object of the location", nullable = true)
    private final Object geojson;

    @Schema(description = "street address of the location", nullable = true)
    private final String streetAddress;

    @Schema(description = "room information of the location", nullable = true)
    private final String room;

    @Schema(description = "postal code of the location", nullable = true)
    private final String postalCode;

    @Schema(description = "lower rank locality of the location", nullable = true)
    private final String subLocality;

    @Schema(description = "locality of the location", nullable = true)
    private final String locality;

    @Schema(description = "List of URLS to bodies this location is used for", nullable = true)
    private final List<String> bodies;

    @Schema(description = "List of URLS to organizations this location is used for", nullable = true)
    private final List<String> organizations;

    @Schema(description = "List of URLS to persons this location is used for", nullable = true)
    private final List<String> persons;

    @Schema(description = "List of URLS to meetings this location is used for", nullable = true)
    private final List<String> meetings;

    @Schema(description = "List of URLS to papers this location is used for", nullable = true)
    private final List<String> papers;
}
