package de.twomartens.oparlservice.entity;

import com.fasterxml.jackson.annotation.JsonFormat;
import com.fasterxml.jackson.annotation.JsonInclude;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.*;

import java.time.LocalDate;
import java.time.ZonedDateTime;
import java.util.List;

@Builder
@Getter
@ToString
@EqualsAndHashCode
@AllArgsConstructor(access = AccessLevel.PRIVATE)
@NoArgsConstructor(force = true, access = AccessLevel.PRIVATE)
@JsonInclude(JsonInclude.Include.NON_NULL)
public class Paper {
    @NonNull
    @Schema(required = true, description = "The unique identifier of the object")
    private final String id;

    @NonNull
    @Schema(required = true, description = "Namespace URL to the object type schema")
    private final String type;

    @NonNull
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @Schema(required = true, description = "Date and time of the creation of the object")
    private final ZonedDateTime created;

    @NonNull
    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @Schema(required = true, description = "Date and time of the last update of the object")
    private final ZonedDateTime modified;

    @Schema(description = "URL to the license", nullable = true)
    private String license;

    @Schema(description = "Categorization of the object", nullable = true)
    private String keyword;

    @Schema(description = "URL of a website that presents the object in the browser", nullable = true)
    private String web;

    @Schema(description = "True, if the object was deleted")
    private final boolean deleted;

    @Schema(description = "URL to the body this paper belongs to", nullable = true)
    private final String body;

    @Schema(description = "name of this paper", nullable = true)
    private final String name;

    @Schema(description = "official identifier to uniquely identify this paper", nullable = true)
    private final String reference;

    @JsonFormat(shape = JsonFormat.Shape.STRING)
    @Schema(description = "date which is used, for example, as a reference for deadlines", nullable = true)
    private final LocalDate date;

    @Schema(description = "type of this paper", nullable = true, example = "Beantwortung einer Anfrage")
    private final String paperType;

    @Schema(description = "List of URLs to related papers", nullable = true)
    private final List<String> relatedPaper;

    @Schema(description = "List of URLs to superordinate papers", nullable = true)
    private final List<String> superordinatedPaper;

    @Schema(description = "List of URLs to subordinate papers", nullable = true)
    private final List<String> subordinatedPapers;

    @Schema(description = "main file for this paper", nullable = true)
    private final File mainFile;

    @Schema(description = "additional files for this paper with exception of mainFile", nullable = true)
    private final List<File> auxiliaryFile;

    @Schema(description = "List of locations related to this paper", nullable = true)
    private final List<Location> location;

    @Schema(description = "List of URLs to authors of this paper", nullable = true)
    private final List<String> originatorPerson;

    @Schema(description = "List of URLs to organizations that authored this paper", nullable = true)
    private final List<String> originatorOrganization;

    @Schema(description = "List of URLs to organizations responsible for content of or response to this paper", nullable = true)
    private final List<String> underDirectionOf;

    @Schema(description = "List of consultations for this paper", nullable = true)
    private final List<Consultation> consultation;
}
